Package: WebGestaltR
Type: Package
Title: Gene Set Analysis Toolkit WebGestaltR
Version: 0.4.6
Date: 2023-05-31
Authors@R: c(
        person("Jing", "Wang", email = "jingwang.uestc@gmail.com", role = "aut"),
        person("Yuxing", "Liao", email = "yuxingliao@gmail.com", role = c("aut", "cre")),
        person("Eric", "Jaehnig", email = "Eric.Jaehnig@bcm.edu", role = c("ctb")),
        person("Zhiao", "Shi", email = "Zhiao.Shi@bcm.edu", role = c("ctb")),
        person("Quanhu", "Sheng", email = "shengqh@gmail.com", role = c("ctb"))
        )
Description: The web version WebGestalt <https://www.webgestalt.org> supports 12 organisms, 354 gene identifiers and 321,251 function categories. Users can upload the data and functional categories with their own gene identifiers. In addition to the Over-Representation Analysis, WebGestalt also supports Gene Set Enrichment Analysis and Network Topology Analysis. The user-friendly output report allows interactive and efficient exploration of enrichment results. The WebGestaltR package not only supports all above functions but also can be integrated into other pipeline or simultaneously analyze multiple gene lists.
License: LGPL
URL: https://github.com/bzhanglab/WebGestaltR
LazyLoad: yes
Depends: R (>= 3.3)
Imports: methods, dplyr, doRNG, readr, parallel (>= 3.3.2), doParallel
        (>= 1.0.10), foreach (>= 1.4.0), jsonlite, httr, rlang,
        svglite, igraph, whisker, apcluster, Rcpp
NeedsCompilation: yes
LinkingTo: Rcpp
RoxygenNote: 7.2.3
Packaged: 2023-06-01 15:42:39 UTC; Yuxing
Author: Jing Wang [aut],
  Yuxing Liao [aut, cre],
  Eric Jaehnig [ctb],
  Zhiao Shi [ctb],
  Quanhu Sheng [ctb]
Maintainer: Yuxing Liao <yuxingliao@gmail.com>
Repository: CRAN
Date/Publication: 2023-06-01 16:00:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 12:38:39 UTC; unix
Archs: WebGestaltR.so.dSYM
