# Generated by roxygen2: do not edit by hand

export(GOSlimSummary)
export(IDMapping)
export(WebGestaltR)
export(WebGestaltRBatch)
export(WebGestaltR_batch)
export(affinityPropagation)
export(formatCheck)
export(goSlimSummary)
export(idMapping)
export(idToSymbol)
export(listArchiveURL)
export(listArchiveUrl)
export(listGeneSet)
export(listIDType)
export(listIdType)
export(listOrganism)
export(listReferenceSet)
export(loadGeneSet)
export(prepareGseaInput)
export(prepareInputMatrixGsea)
export(readGmt)
export(swGsea)
export(weightedSetCover)
import(grDevices)
import(graphics)
import(methods)
import(utils)
importFrom(Rcpp,sourceCpp)
importFrom(apcluster,apcluster)
importFrom(doParallel,registerDoParallel)
importFrom(doRNG,"%dorng%")
importFrom(dplyr,"%>%")
importFrom(dplyr,arrange)
importFrom(dplyr,bind_rows)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,right_join)
importFrom(dplyr,select)
importFrom(foreach,"%dopar%")
importFrom(foreach,foreach)
importFrom(httr,GET)
importFrom(httr,POST)
importFrom(httr,content)
importFrom(httr,modify_url)
importFrom(igraph,V)
importFrom(igraph,get.adjacency)
importFrom(igraph,graph.edgelist)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(parallel,makeCluster)
importFrom(parallel,mclapply)
importFrom(parallel,stopCluster)
importFrom(readr,cols)
importFrom(readr,read_tsv)
importFrom(readr,stop_for_problems)
importFrom(readr,write_tsv)
importFrom(rlang,.data)
importFrom(stats,IQR)
importFrom(stats,median)
importFrom(stats,p.adjust)
importFrom(stats,phyper)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,sd)
importFrom(svglite,svglite)
importFrom(tools,file_ext)
importFrom(whisker,rowSplit)
importFrom(whisker,whisker.render)
useDynLib(WebGestaltR)
