# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Fill relation data frame for GSEA input
#'
#' Fill 1 for gene in gene set
#'
#' @param gmt A Data Frame with geneSet and gene columns from the GMT file
#' @param genes A vector of genes
#' @param geneSets A vector of gene sets
#'
#' @return A Data Frame with the first column of gene and 1 or 0 for other columns of gene sets.
#' @author Yuxing Liao
#' @keywords internal
fillInputDataFrame <- function(gmt, genes, geneSets) {
    .Call('_WebGestaltR_fillInputDataFrame', PACKAGE = 'WebGestaltR', gmt, genes, geneSets)
}

#' Permutaion in GSEA algorithm
#'
#' @param inset_scores Scaled score matrix for genes in sets
#' @param outset_scores Normalized score matrix for genes not in sets
#' @param expression_value Vector of gene rank scores
#'
#' @return A vector of concatenated random minimal,maimum and best running sum scores for each set.
#' @author Yuxing Liao
#' @keywords internal
gseaPermutation <- function(inset_scores, outset_scores, expression_value) {
    .Call('_WebGestaltR_gseaPermutation', PACKAGE = 'WebGestaltR', inset_scores, outset_scores, expression_value)
}

