% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reportUtils.R
\name{keepRep}
\alias{keepRep}
\title{keepRep}
\usage{
keepRep(topResult, allResult, reps)
}
\description{
Add representatives of redundancy-reduced clusters to
topResult if they are missing.
}
\keyword{internal}
